#include <stdlib.h>
#include <fstream>
#include "SDL_helper.hpp"
#include "Board.hpp"
#include "Piece.hpp"
#include "Level.hpp"

Board::Board()
{   
   level = 0;
   speed = 0;
   lines = 0;
   score = 0;
   ready = 10;
   preview = new Piece();
   current = new Piece();

   for( int y=0;y<25;y++)
     for( int x=0;x<12;x++)
       tab[y][x] = 0;
}

Board::~Board()
{   
   delete preview;
   delete current;
}

int Board::LoadLevel( int nlevel) 
{      
   level = nlevel;
   speed = nlevel;
   current_level = GetLevel( nlevel);
   
   // Set type of piece
   current->SetType( (random()%7)+1);
   preview->SetType( (random()%7)+1);
   
   //sfx_pt[0] = VSDL_LoadSFX( "brik.wav");
   //sfx_pt[1] = VSDL_LoadSFX( "line.wav");
   
   return 1;
}

void Board::Display()
{
   int j, i;
   char buf[256];

   // Draw background
   
   SDL_Rect dest;
   dest.x = 0;
   dest.y = 0;
   dest.w = current_level->background->w;
   dest.h = current_level->background->h;
   SDL_BlitSurface( current_level->background, NULL, SDL_GetVideoSurface(), &dest);
   
   for(j=0;j<24;j++)
     for(i=0;i<12;i++)
       if ( tab[j][i] != 0 )
	 {	 
	    SDL_Rect dest;
	    dest.x = 200+i*20;
	    dest.y = j*20;
	    dest.w = current_level->piece_sprite->w;
	    dest.h = current_level->piece_sprite->h;
	    SDL_BlitSurface( current_level->piece_sprite, NULL, SDL_GetVideoSurface(), &dest);
	 }
   
   current->Display( 200, 0,   current_level->piece_sprite);
   preview->Display( 440, 360, current_level->piece_sprite);
   
   VSDL_Write( 480, 100, "Score", 0) ;
   sprintf( buf, "%d", score) ;
   VSDL_Write( 480, 130, buf , 0) ;
   VSDL_Write( 480, 160, "Lines", 0) ;
   sprintf( buf, "%d", lines) ;
   VSDL_Write( 480, 190, buf , 0) ;
   VSDL_Write( 480, 220, "Level", 0) ;
   sprintf( buf, "%d", level) ;
   VSDL_Write( 480, 250, buf , 0) ;
   
/*   for(i=0;i<MAX_SFX;i++) 
     {   
	if ( sfx[i] > 0 ) {
	   VSDL_PlaySFX( sfx_pt[i]) ;
	   sfx[i] = 0 ;
	}
     }*/
}

int Board::Update()
{
   int tmp;
   
   tmp = 0;
   ready -= 1;
   if ( ready < speed ) 
     {
	ready = 10;
	speed = level;
	current->Move( 4); /* Move down */
	
	if ( DetectCollision( current) == 1 ) 
	  {
	     current->Undo() ;
	     FixPiece( current) ;
	     //sfx[0] = 1 ;
	     score = score + 2 ;
	     tmp = WatchForLines() ;
	     lines = lines + tmp ;
	     score = score + tmp*tmp*100 ;
	     
	     if ( score > ((2^level)*1000) ) {
		LoadLevel( level++) ;
	     }
	     //if ( tmp > 0 )
	       //sfx[1] = 1 ;
	     tmp = WatchForLose() ;
	  }
     }
   return tmp;
}

int Board::DetectCollision( Piece* piece)
{
   int j, i ;   
   
   for(j=0;j<4;j++) {
      for(i=0;i<4;i++) {
	 if ( piece->shape[j*4+i] == '1' ) { 
	    if ( piece->posY+j >= 24 ) { /* la piece est en bas */
	       return 1;
	    }
	    else if ( piece->posX+i < 0 || piece->posX+i > 11 ) { /* collision avec les bords */
	       return 1;
	    }
	    else if ( tab[piece->posY+j][piece->posX+i] != 0  ) { /* collision */
	       return 1;	       
	    }  
	 }
      }
   }
   return 0;
}

void Board::FixPiece(Piece* piece)
{
   int j, i ;

   for(j=0;j<4;j++) {
      for(i=0;i<4;i++) {
	 if ( current->shape[j*4+i] == '1' ) {
	    tab[current->posY+j][current->posX+i] = 1 ;
	 }
      }
   }
   current->SetType( preview->type) ;
   current->posX = 6 ;
   current->posY = -2 ;
   preview->SetType((random()%7)+1) ;
}
   
int Board::WatchForLines()
{
   int j, i ;
   int dec ;
   int nbBriks, nbLines ;
   
   nbLines = 0 ;
   for(j=0;j<24;j++) {
      nbBriks = 0 ;
      for(i=0;i<12;i++) {
	 if ( tab[j][i] != 0 )
	   nbBriks++ ;
      }
      if ( nbBriks == 12 ) { /* line completed */
	 nbLines++ ;
	 for(dec=j;dec>1;dec--)
	   for(i=0;i<12;i++)
	     tab[dec][i] = tab[dec-1][i] ;
      }
   }
   return( nbLines) ;
}

int Board::WatchForLose() 
{
   int i ;
   
   for(i=0;i<12;i++)
     if ( tab[1][i] != 0 )
       return( 1) ;
   return( 0) ;
}

void Board::MovePlayer( int direction)
{
   
   if ( direction & MOVE_ROT ) {
      current->Move( MOVE_ROT) ;
      if ( DetectCollision( current) == 1 )
	current->Undo() ;
   }
   if (direction & MOVE_RIGHT ) {
      current->Move( MOVE_RIGHT) ;
      if ( DetectCollision( current) == 1 )
	current->Undo() ;
   }
   if ( direction & MOVE_DOWN ) {
      current->Move( MOVE_DOWN) ;
            if ( DetectCollision( current) == 1 )
	        current->Undo() ;
      speed = 10 ;
   }
   if ( direction & MOVE_LEFT ) {
      current->Move( MOVE_LEFT) ;
      if ( DetectCollision( current) == 1 )
	current->Undo() ;
   }
}
