#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <SDL.h>
#include <SDL_image.h>
#include "SDL_helper.hpp"

VSDL_Fonts *vsdl_currentFonts;

Uint32 waitFrame( Uint32 fps)
{
   static Uint32 next_tick=0;
   Uint32 this_tick;
   
   this_tick = SDL_GetTicks();
   
   if ( this_tick < next_tick ) 
     {
	SDL_Delay(next_tick-this_tick);
     }
   
   next_tick = this_tick + (1000/fps); 
   return 0; // return the real fps
}


SDL_Surface *VSDL_LoadImage( char* filename, int tr, int alpha)
{
   
   SDL_RWops *rw = SDL_RWFromFile( filename, "r");
   
   if ( rw == NULL ) 
     {
	fprintf( stderr, "Couldn't load %s: %s\n", filename, SDL_GetError());
	return NULL;
     }
   
   return VSDL_LoadImage_RW( rw, tr, alpha);
}


SDL_Surface *VSDL_LoadImage_RW( SDL_RWops *rw, int tr, int alpha)
{
   
   SDL_Surface *new_surface = IMG_Load_RW( rw, 0);
   
   if ( new_surface == NULL ) 
     {
	fprintf( stderr, "Couldn't load image: %s\n", SDL_GetError());
	return NULL;
     }
   if ( tr == 1 ) 
     { 
	// Set transparency
	Uint32 trColor ;
	
	trColor = SDL_MapRGB( new_surface->format, 0, 0, 0);
	SDL_SetColorKey( new_surface, SDL_SRCCOLORKEY|SDL_RLEACCEL, trColor);
     }
   if ( alpha != 0 ) 
     { 
	// Set alpha 
	SDL_SetAlpha( new_surface, SDL_SRCALPHA, alpha );
     }
   
   SDL_Surface *surface = SDL_DisplayFormat( new_surface);
   SDL_FreeSurface( new_surface);
   
   return surface;
}

int VSDL_BlitSurface( SDL_Surface* surface, int x, int y) 
{
   SDL_Rect dest;
   dest.x = x;
   dest.y = y;
   dest.w = surface->w;
   dest.h = surface->h;
   return SDL_BlitSurface( surface, NULL, SDL_GetVideoSurface(), &dest);;
}

/* load fonts from file */
VSDL_Fonts* VSDL_LoadFonts( char* filename, int fw, int fh )
{
   SDL_RWops *rw = SDL_RWFromFile( filename, "r");
   
   if ( rw == NULL ) 
     {
	fprintf( stderr, "Couldn't load font %s: %s\n", filename, SDL_GetError());
	return NULL;
     }
      
   return VSDL_LoadFonts_RW( rw, fw, fh);
}

  
/* load fonts from file */
VSDL_Fonts* VSDL_LoadFonts_RW( SDL_RWops *rw, int fw, int fh )
{	       
   // Load a pixmap file which is the size of  w*h
   // the letters must be:
   // line 0: abc...z
   // line 1: ABC...Z
   // line 2: 0123456789 and the ".,:!?()"
   
   VSDL_Fonts* nfonts = new VSDL_Fonts;
   nfonts->w = fw;
   nfonts->h = fh;
   nfonts->surface = VSDL_LoadImage_RW( rw, 1);
  /*  
   for(i=0;i<26;i++) {
      nfonts->font[i] = VSDL_CopyPiece( tmp, i*fw, 0, fw, fh) ; 
   }
   for(i=0;i<26;i++) {
      nfonts->font[i+26] = VSDL_CopyPiece( tmp, i*fw, fh, fw, fh) ;
   }
   for(i=0;i<20;i++) { 
      nfonts->font[i+52] = VSDL_CopyPiece( tmp, i*fw, fh*2, fw, fh) ;
   }
  */
   return nfonts;
}

/* Set the cuurent fonts */
int VSDL_SetFonts( VSDL_Fonts* fonts)
{
   
   vsdl_currentFonts = fonts ;
 
   return 1;
}


/* Write a string with fonts */
int VSDL_Write( int x, int y, char* txt, const int flags=0)
{
   int i;
   int lng; 
   int row, line; 
   int ok;

   SDL_Surface *vsdl_screen = SDL_GetVideoSurface();
   
   lng = strlen(txt);
   if ( flags && VSDL_CENTERED ) 
     {
	x = x + (  vsdl_screen->w - vsdl_currentFonts->w * lng ) / 2 ;
     }
   for(i=0;i<lng;i++) {
      ok = 0 ;
      row = -1 ;
      if ( txt[i] >= 65 && txt[i] <= 65+26 ) { // is an "ABC...Z" letter
	 row = txt[i] - 65;
	 line = 0;
	 ok = 1 ;
      }
      else if ( txt[i] >= 97 && txt[i] <= 97+26 ) { // is an "abc...z" letter
	 row = txt[i] - 97;// + 26;
	 line = 1;
	 ok = 1;
      }
      else if ( txt[i] >= 48 && txt[i] <= 48+20 ) { // is an "012...@" letter
	 row = txt[i] - 48;// + 52;
	 line = 2;
	 ok = 1;
      }
      if ( ok == 1 )
	{
	   SDL_Rect dest;
	   dest.x = x+i*vsdl_currentFonts->w;
	   dest.y = y;
	   dest.w = vsdl_currentFonts->w;
	   dest.h = vsdl_currentFonts->h;

	   SDL_Rect src;
	   src.x = row*vsdl_currentFonts->w;
	   src.y = line*vsdl_currentFonts->h;
	   src.w = vsdl_currentFonts->w;
	   src.h = vsdl_currentFonts->h;
	   
	   SDL_BlitSurface( vsdl_currentFonts->surface, &src, vsdl_screen, &dest);     
	}
   }
   return 1;
}

